@@ -12,8 +12,8 @@
 
  /* see bottom of file for comments on post processing */
 
-#include <stdarg.h>
 #include <memory.h>
+#include <stdint.h>
 
 #include "post-process.h"
 
@@ -76,14 +76,14 @@ static int string_in_list(const char * s, const char * a[])
  * Return the name of the domain associated with the provided starting
  * link. Return -1 if link isn't associated with a domain.
  */
-static int find_domain_name(Postprocessor *pp, const char *link)
+static size_t find_domain_name(Postprocessor *pp, const char *link)
 {
-	int i,domain;
+	size_t i, domain;
 	StartingLinkAndDomain *sllt = pp->knowledge->starting_link_lookup_table;
 	for (i=0;;i++)
 	{
 		domain = sllt[i].domain;
-		if (domain==-1) return -1;					/* hit the end-of-list sentinel */
+		if (domain == SIZE_MAX) return SIZE_MAX;  /* hit the end-of-list sentinel */
 		if (post_process_match(sllt[i].starting_link, link)) return domain;
 	}
 }
@@ -92,9 +92,9 @@ static int find_domain_name(Postprocessor *pp, const char *link)
 static int contained_in(const Domain * d1, const Domain * d2,
                         const Sublinkage *sublinkage)
 {
-	char mark[MAX_LINKS];
+	bool mark[sublinkage->num_links];
 	List_o_links * lol;
-	memset(mark, 0, sublinkage->num_links*(sizeof mark[0]));
+	memset(mark, 0, sublinkage->num_links*(sizeof(bool)));
 	for (lol=d2->lol; lol != NULL; lol = lol->next)
 		mark[lol->link] = TRUE;
 	for (lol=d1->lol; lol != NULL; lol = lol->next)
@@ -103,7 +103,7 @@ static int contained_in(const Domain * d1, const Domain * d2,
 }
 
 /** Returns the predicate "the given link is in the given domain" */
-static int link_in_domain(int link, const Domain * d)
+static bool link_in_domain(size_t link, const Domain * d)
 {
 	List_o_links * lol;
 	for (lol = d->lol; lol != NULL; lol = lol->next)
@@ -117,7 +117,7 @@ static int link_in_domain(int link, const Domain * d)
 /* Although this is no longer used, I'm leaving the code here for future reference --DS 3/98 */
 
 /* Returns TRUE if the domains actually form a properly nested structure */
-static int check_domain_nesting(Postprocessor *pp, int num_links)
+static bool check_domain_nesting(Postprocessor *pp, int num_links)
 {
 	Domain * d1, * d2;
 	int counts[4];
@@ -175,7 +175,7 @@ static void free_D_tree_leaves(DTreeLeaf *dtl)
  */
 void post_process_free_data(PP_data * ppd)
 {
-	int w, d;
+	size_t w, d;
 	for (w = 0; w < ppd->length; w++)
 	{
 		free_List_o_links(ppd->word_links[w]);
@@ -220,9 +220,9 @@ static void mark_reachable_words(Postprocessor *pp, int w)
  * Returns true if the linkage is connected, considering words
  * that have at least one edge....this allows conjunctive sentences
  * not to be thrown out. */
-static int is_connected(Postprocessor *pp)
+static bool is_connected(Postprocessor *pp)
 {
-	int i;
+	size_t i;
 	for (i=0; i<pp->pp_data.length; i++)
 		pp->visited[i] = (pp->pp_data.word_links[i] == NULL);
 	mark_reachable_words(pp, 0);
@@ -235,7 +235,7 @@ static int is_connected(Postprocessor *pp)
 static void build_type_array(Postprocessor *pp)
 {
 	D_type_list * dtl;
-	int d;
+	size_t d;
 	List_o_links * lol;
 	for (d=0; d<pp->pp_data.N_domains; d++)
 	{
@@ -313,21 +313,21 @@ static void reset_pp_node(Postprocessor *pp)
 
 /************************ rule application *******************************/
 
-static int apply_rules(Postprocessor *pp,
-							 int (applyfn) (Postprocessor *,Sublinkage *,pp_rule *),
+static bool apply_rules(Postprocessor *pp,
+							 bool (applyfn) (Postprocessor *, Sublinkage *, pp_rule *),
 							 Sublinkage *sublinkage,
 							 pp_rule *rule_array,
 							 const char **msg)
 {
 	int i;
 	for (i=0; (*msg=rule_array[i].msg)!=NULL; i++)
-		if (!applyfn(pp, sublinkage, &(rule_array[i]))) return 0;
-	return 1;
+		if (!applyfn(pp, sublinkage, &(rule_array[i]))) return false;
+	return true;
 }
 
-static int
+static bool
 apply_relevant_rules(Postprocessor *pp,
-						 int(applyfn)(Postprocessor *pp,Sublinkage*,pp_rule *rule),
+						 bool(applyfn)(Postprocessor *, Sublinkage*, pp_rule *),
 						 Sublinkage *sublinkage,
 						 pp_rule *rule_array,
 						 int *relevant_rules,
@@ -343,10 +343,10 @@ apply_relevant_rules(Postprocessor *pp,
 
 	/* we did, and we don't */
 	for (i=0; (idx=relevant_rules[i])!=-1; i++) {
-			*msg = rule_array[idx].msg;	 /* Adam had forgotten this -- DS 4/9/98 */
-			if (!applyfn(pp, sublinkage, &(rule_array[idx]))) return 0;
+			*msg = rule_array[idx].msg;
+			if (!applyfn(pp, sublinkage, &(rule_array[idx]))) return false;
 	}
-	return 1;
+	return true;
 }
 
 /**
@@ -354,24 +354,24 @@ apply_relevant_rules(Postprocessor *pp,
  * contain at least one from the required list.	(as determined by exact
  * string matching)
  */
-static int
+static bool
 apply_contains_one(Postprocessor *pp, Sublinkage *sublinkage, pp_rule *rule)
 {
 	DTreeLeaf * dtl;
-	int d, count;
+	size_t d, count;
 	for (d=0; d<pp->pp_data.N_domains; d++)
 		{
 			for (dtl = pp->pp_data.domain_array[d].child;
 			 dtl != NULL &&
 				 !post_process_match(rule->selector,
-								 sublinkage->link[dtl->link]->name);
+								 sublinkage->link[dtl->link]->link_name);
 			 dtl = dtl->next) {}
 			if (dtl != NULL)
 		{
 			/* selector link of rule appears in this domain */
 			count=0;
 			for (dtl = pp->pp_data.domain_array[d].child; dtl != NULL; dtl = dtl->next)
-				if (string_in_list(sublinkage->link[dtl->link]->name,
+				if (string_in_list(sublinkage->link[dtl->link]->link_name,
 									 rule->link_array))
 					{
 				count=1;
@@ -389,27 +389,27 @@ apply_contains_one(Postprocessor *pp, Sublinkage *sublinkage, pp_rule *rule)
  * all groups containing the selector link do not contain anything
  * from the link_array contained in the rule. Uses exact string matching.
  */
-static int
+static bool
 apply_contains_none(Postprocessor *pp,Sublinkage *sublinkage,pp_rule *rule)
 {
-	DTreeLeaf * dtl;
-	int d;
+	size_t d;
 	for (d=0; d<pp->pp_data.N_domains; d++)
-		{
-			for (dtl = pp->pp_data.domain_array[d].child;
+	{
+		DTreeLeaf * dtl;
+		for (dtl = pp->pp_data.domain_array[d].child;
 			 dtl != NULL &&
 				 !post_process_match(rule->selector,
-								 sublinkage->link[dtl->link]->name);
+								 sublinkage->link[dtl->link]->link_name);
 			 dtl = dtl->next) {}
-			if (dtl != NULL)
+		if (dtl != NULL)
 		{
 			/* selector link of rule appears in this domain */
 			for (dtl = pp->pp_data.domain_array[d].child; dtl != NULL; dtl = dtl->next)
-				if (string_in_list(sublinkage->link[dtl->link]->name,
+				if (string_in_list(sublinkage->link[dtl->link]->link_name,
 									 rule->link_array))
 					return FALSE;
 		}
-		}
+	}
 	return TRUE;
 }
 
@@ -418,36 +418,38 @@ apply_contains_none(Postprocessor *pp,Sublinkage *sublinkage,pp_rule *rule)
  * (1) the sentence doesn't contain the selector link for the rule, or
  * (2) it does, and it also contains one or more from the rule's link set
  */
-static int
+static bool
 apply_contains_one_globally(Postprocessor *pp,Sublinkage *sublinkage,pp_rule *rule)
 {
-	int i,j,count;
+	size_t i,j,count;
 	for (i=0; i<sublinkage->num_links; i++) {
-		if (sublinkage->link[i]->l == -1) continue;
-		if (post_process_match(rule->selector,sublinkage->link[i]->name)) break;
+		// if (sublinkage->link[i]->lw == SIZE_MAX) continue;
+		assert (sublinkage->link[i]->lw != SIZE_MAX);
+		if (post_process_match(rule->selector,sublinkage->link[i]->link_name)) break;
 	}
-	if (i==sublinkage->num_links) return TRUE;
+	if (i == sublinkage->num_links) return TRUE;
 
 	/* selector link of rule appears in sentence */
-	count=0;
+	count = 0;
 	for (j=0; j<sublinkage->num_links && count==0; j++) {
-		if (sublinkage->link[j]->l == -1) continue;
-		if (string_in_list(sublinkage->link[j]->name, rule->link_array))
-			{
-		count=1;
-		break;
-			}
+		// if (sublinkage->link[j]->lw == SIZE_MAX) continue;
+		assert (sublinkage->link[j]->lw != SIZE_MAX);
+		if (string_in_list(sublinkage->link[j]->link_name, rule->link_array))
+		{
+			count=1;
+			break;
+		}
 	}
-	if (count==0) return FALSE; else return TRUE;
+	if (count == 0) return FALSE; else return TRUE;
 }
 
-static int
+static bool
 apply_connected(Postprocessor *pp, Sublinkage *sublinkage, pp_rule *rule)
 {
 	/* There is actually just one (or none, if user didn't specify it)
 		 rule asserting that linkage is connected. */
-	if (!is_connected(pp)) return 0;
-	return 1;
+	if (!is_connected(pp)) return false;
+	return true;
 }
 
 #if 0
@@ -478,7 +480,8 @@ apply_connected_without(Postprocessor *pp,Sublinkage *sublinkage,pp_rule *rule)
 	 end points of that link are still connected.
 */
 
-static void reachable_without_dfs(Postprocessor *pp, Sublinkage *sublinkage, int a, int b, int w)
+static void reachable_without_dfs(Postprocessor *pp,
+                Sublinkage *sublinkage, size_t a, size_t b, size_t w)
 {
 	/* This is a depth first search of words reachable from w, excluding
 	 * any direct edge between word a and word b. */
@@ -500,17 +503,17 @@ static void reachable_without_dfs(Postprocessor *pp, Sublinkage *sublinkage, int
  * of the subgraph reachable from word 0 changes as a result of deleting
  * these links.
  */
-static int
-apply_must_form_a_cycle(Postprocessor *pp,Sublinkage *sublinkage,pp_rule *rule)
+static bool
+apply_must_form_a_cycle(Postprocessor *pp,Sublinkage *sublinkage, pp_rule *rule)
 {
 	List_o_links *lol;
-	int w;
+	size_t w;
 	for (w=0; w<pp->pp_data.length; w++)
 	{
 		for (lol = pp->pp_data.word_links[w]; lol != NULL; lol = lol->next)
 		{
 			if (w > lol->word) continue;	/* only consider each edge once */
-			if (!pp_linkset_match(rule->link_set, sublinkage->link[lol->link]->name)) continue;
+			if (!pp_linkset_match(rule->link_set, sublinkage->link[lol->link]->link_name)) continue;
 			memset(pp->visited, 0, pp->pp_data.length*(sizeof pp->visited[0]));
 			reachable_without_dfs(pp, sublinkage, w, lol->word, w);
 			if (!pp->visited[lol->word]) return FALSE;
@@ -519,9 +522,9 @@ apply_must_form_a_cycle(Postprocessor *pp,Sublinkage *sublinkage,pp_rule *rule)
 
 	for (lol = pp->pp_data.links_to_ignore; lol != NULL; lol = lol->next)
 	{
-		w = sublinkage->link[lol->link]->l;
+		w = sublinkage->link[lol->link]->lw;
 		/* (w, lol->word) are the left and right ends of the edge we're considering */
-		if (!pp_linkset_match(rule->link_set, sublinkage->link[lol->link]->name)) continue;
+		if (!pp_linkset_match(rule->link_set, sublinkage->link[lol->link]->link_name)) continue;
 		memset(pp->visited, 0, pp->pp_data.length*(sizeof pp->visited[0]));
 		reachable_without_dfs(pp, sublinkage, w, lol->word, w);
 		if (!pp->visited[lol->word]) return FALSE;
@@ -537,19 +540,20 @@ apply_must_form_a_cycle(Postprocessor *pp,Sublinkage *sublinkage,pp_rule *rule)
  * all of the words that touch a link in the domain are not to the left
  * of the root word of the domain.
  */
-static int
+static bool
 apply_bounded(Postprocessor *pp, Sublinkage *sublinkage, pp_rule *rule)
 {
-	int d, lw, d_type;
+	size_t d, lw;
+	char d_type;
 	List_o_links * lol;
 	d_type = rule->domain;
 	for (d=0; d<pp->pp_data.N_domains; d++)
 	{
 		if (pp->pp_data.domain_array[d].type != d_type) continue;
-		lw = sublinkage->link[pp->pp_data.domain_array[d].start_link]->l;
+		lw = sublinkage->link[pp->pp_data.domain_array[d].start_link]->lw;
 		for (lol = pp->pp_data.domain_array[d].lol; lol != NULL; lol = lol->next)
 		{
-			if (sublinkage->link[lol->link]->l < lw) return FALSE;
+			if (sublinkage->link[lol->link]->lw < lw) return FALSE;
 		}
 	}
 	return TRUE;
@@ -563,7 +567,7 @@ apply_bounded(Postprocessor *pp, Sublinkage *sublinkage, pp_rule *rule)
  */
 static void build_graph(Postprocessor *pp, Sublinkage *sublinkage)
 {
-	int i, link;
+	size_t i, link;
 	List_o_links * lol;
 
 	for (i=0; i<pp->pp_data.length; i++)
@@ -571,28 +575,29 @@ static void build_graph(Postprocessor *pp, Sublinkage *sublinkage)
 
 	for (link=0; link<sublinkage->num_links; link++)
 	{
-		if (sublinkage->link[link]->l == -1) continue;
-		if (pp_linkset_match(pp->knowledge->ignore_these_links, sublinkage->link[link]->name))
+		// if (sublinkage->link[link]->lw == SIZE_MAX) continue;
+		assert (sublinkage->link[link]->lw != SIZE_MAX);
+		if (pp_linkset_match(pp->knowledge->ignore_these_links, sublinkage->link[link]->link_name))
 		{
 			lol = (List_o_links *) xalloc(sizeof(List_o_links));
 			lol->next = pp->pp_data.links_to_ignore;
 			pp->pp_data.links_to_ignore = lol;
 			lol->link = link;
-			lol->word = sublinkage->link[link]->r;
+			lol->word = sublinkage->link[link]->rw;
 			continue;
 		}
 
 		lol = (List_o_links *) xalloc(sizeof(List_o_links));
-		lol->next = pp->pp_data.word_links[sublinkage->link[link]->l];
-		pp->pp_data.word_links[sublinkage->link[link]->l] = lol;
+		lol->next = pp->pp_data.word_links[sublinkage->link[link]->lw];
+		pp->pp_data.word_links[sublinkage->link[link]->lw] = lol;
 		lol->link = link;
-		lol->word = sublinkage->link[link]->r;
+		lol->word = sublinkage->link[link]->rw;
 
 		lol = (List_o_links *) xalloc(sizeof(List_o_links));
-		lol->next = pp->pp_data.word_links[sublinkage->link[link]->r];
-		pp->pp_data.word_links[sublinkage->link[link]->r] = lol;
+		lol->next = pp->pp_data.word_links[sublinkage->link[link]->rw];
+		pp->pp_data.word_links[sublinkage->link[link]->rw] = lol;
 		lol->link = link;
-		lol->word = sublinkage->link[link]->l;
+		lol->word = sublinkage->link[link]->lw;
 	}
 }
 
@@ -618,7 +623,7 @@ static void add_link_to_domain(Postprocessor *pp, int link)
 }
 
 static void depth_first_search(Postprocessor *pp, Sublinkage *sublinkage,
-									 int w, int root,int start_link)
+									 size_t w, size_t root, size_t start_link)
 {
 	List_o_links *lol;
 	pp->visited[w] = TRUE;
@@ -631,13 +636,13 @@ static void depth_first_search(Postprocessor *pp, Sublinkage *sublinkage,
 		if (!pp->visited[lol->word] && (lol->word != root) &&
 		!(lol->word < root && lol->word < w &&
 					pp_linkset_match(pp->knowledge->restricted_links,
-							 sublinkage->link[lol->link]->name)))
+							 sublinkage->link[lol->link]->link_name)))
 			depth_first_search(pp, sublinkage, lol->word, root, start_link);
 	}
 }
 
 static void bad_depth_first_search(Postprocessor *pp, Sublinkage *sublinkage,
-									 int w, int root, int start_link)
+                                 size_t w, size_t root, size_t start_link)
 {
 	List_o_links * lol;
 	pp->visited[w] = TRUE;
@@ -650,13 +655,13 @@ static void bad_depth_first_search(Postprocessor *pp, Sublinkage *sublinkage,
 		if ((!pp->visited[lol->word]) && !(w == root && lol->word < w) &&
 		!(lol->word < root && lol->word < w &&
 					pp_linkset_match(pp->knowledge->restricted_links,
-							 sublinkage->link[lol->link]->name)))
+							 sublinkage->link[lol->link]->link_name)))
 			bad_depth_first_search(pp, sublinkage, lol->word, root, start_link);
 	}
 }
 
 static void d_depth_first_search(Postprocessor *pp, Sublinkage *sublinkage,
-								 int w, int root, int right, int start_link)
+                    size_t w, size_t root, size_t right, size_t start_link)
 {
 	List_o_links * lol;
 	pp->visited[w] = TRUE;
@@ -670,13 +675,13 @@ static void d_depth_first_search(Postprocessor *pp, Sublinkage *sublinkage,
 		!(w == root && lol->word < root) &&
 		!(lol->word < root && lol->word < w &&
 					pp_linkset_match(pp->knowledge->restricted_links,
-							 sublinkage->link[lol->link]->name)))
+							 sublinkage->link[lol->link]->link_name)))
 			d_depth_first_search(pp,sublinkage,lol->word,root,right,start_link);
 	}
 }
 
 static void left_depth_first_search(Postprocessor *pp, Sublinkage *sublinkage,
-															 int w, int right,int start_link)
+                               size_t w, size_t right, size_t start_link)
 {
 	List_o_links *lol;
 	pp->visited[w] = TRUE;
@@ -696,13 +701,14 @@ static int domain_compare(const Domain * d1, const Domain * d2)
 
 static void build_domains(Postprocessor *pp, Sublinkage *sublinkage)
 {
-	int link, i, d;
+	size_t link, i, d;
 	const char *s;
 	pp->pp_data.N_domains = 0;
 
 	for (link = 0; link<sublinkage->num_links; link++) {
-		if (sublinkage->link[link]->l == -1) continue;
-		s = sublinkage->link[link]->name;
+		// if (sublinkage->link[link]->lw == SIZE_MAX) continue;
+		assert (sublinkage->link[link]->lw != SIZE_MAX);
+		s = sublinkage->link[link]->link_name;
 
 		if (pp_linkset_match(pp->knowledge->ignore_these_links, s)) continue;
 		if (pp_linkset_match(pp->knowledge->domain_starter_links, s))
@@ -710,8 +716,8 @@ static void build_domains(Postprocessor *pp, Sublinkage *sublinkage)
 		setup_domain_array(pp, pp->pp_data.N_domains, s, link);
 				if (pp_linkset_match(pp->knowledge->domain_contains_links, s))
 			add_link_to_domain(pp, link);
-		depth_first_search(pp,sublinkage,sublinkage->link[link]->r,
-							 sublinkage->link[link]->l, link);
+		depth_first_search(pp,sublinkage,sublinkage->link[link]->rw,
+							 sublinkage->link[link]->lw, link);
 
 		pp->pp_data.N_domains++;
 				assert(pp->pp_data.N_domains<PP_MAX_DOMAINS, "raise value of PP_MAX_DOMAINS");
@@ -722,8 +728,8 @@ static void build_domains(Postprocessor *pp, Sublinkage *sublinkage)
 			setup_domain_array(pp, pp->pp_data.N_domains, s, link);
 			/* always add the starter link to its urfl domain */
 			add_link_to_domain(pp, link);
-			bad_depth_first_search(pp,sublinkage,sublinkage->link[link]->r,
-								 sublinkage->link[link]->l,link);
+			bad_depth_first_search(pp,sublinkage,sublinkage->link[link]->rw,
+								 sublinkage->link[link]->lw, link);
 			pp->pp_data.N_domains++;
 			assert(pp->pp_data.N_domains<PP_MAX_DOMAINS,"raise PP_MAX_DOMAINS value");
 		}
@@ -732,19 +738,19 @@ static void build_domains(Postprocessor *pp, Sublinkage *sublinkage)
 			{
 				setup_domain_array(pp, pp->pp_data.N_domains, s, link);
 				/* do not add the starter link to its urfl_only domain */
-				d_depth_first_search(pp,sublinkage, sublinkage->link[link]->l,
-								 sublinkage->link[link]->l,
-								 sublinkage->link[link]->r,link);
+				d_depth_first_search(pp,sublinkage, sublinkage->link[link]->lw,
+								 sublinkage->link[link]->lw,
+								 sublinkage->link[link]->rw, link);
 				pp->pp_data.N_domains++;
 				assert(pp->pp_data.N_domains<PP_MAX_DOMAINS,"raise PP_MAX_DOMAINS value");
 			}
 		else
-			if (pp_linkset_match(pp->knowledge->left_domain_starter_links,s))
+			if (pp_linkset_match(pp->knowledge->left_domain_starter_links, s))
 				{
 					setup_domain_array(pp, pp->pp_data.N_domains, s, link);
 					/* do not add the starter link to a left domain */
-					left_depth_first_search(pp,sublinkage, sublinkage->link[link]->l,
-									 sublinkage->link[link]->r,link);
+					left_depth_first_search(pp,sublinkage, sublinkage->link[link]->lw,
+									 sublinkage->link[link]->rw, link);
 					pp->pp_data.N_domains++;
 					assert(pp->pp_data.N_domains<PP_MAX_DOMAINS,"raise PP_MAX_DOMAINS value");
 				}
@@ -760,7 +766,7 @@ static void build_domains(Postprocessor *pp, Sublinkage *sublinkage)
 	/* sanity check: all links in all domains have a legal domain name */
 	for (d=0; d<pp->pp_data.N_domains; d++) {
 		i = find_domain_name(pp, pp->pp_data.domain_array[d].string);
-		if (i == -1)
+		if (i == SIZE_MAX)
 			 prt_error("Error: post_process(): Need an entry for %s in LINK_TYPE_TABLE",
 					 pp->pp_data.domain_array[d].string);
 		pp->pp_data.domain_array[d].type = i;
@@ -769,25 +775,23 @@ static void build_domains(Postprocessor *pp, Sublinkage *sublinkage)
 
 static void build_domain_forest(Postprocessor *pp, Sublinkage *sublinkage)
 {
-	int d, d1, link;
+	size_t d, d1, link;
 	DTreeLeaf * dtl;
 	if (pp->pp_data.N_domains > 0)
+	{
 		pp->pp_data.domain_array[pp->pp_data.N_domains-1].parent = NULL;
-	for (d=0; d < pp->pp_data.N_domains-1; d++) {
-		for (d1 = d+1; d1 < pp->pp_data.N_domains; d1++) {
-			if (contained_in(&pp->pp_data.domain_array[d],&pp->pp_data.domain_array[d1],sublinkage))
-			{
-				pp->pp_data.domain_array[d].parent = &pp->pp_data.domain_array[d1];
-				break;
+		for (d=0; d < pp->pp_data.N_domains-1; d++) {
+			for (d1 = d+1; d1 < pp->pp_data.N_domains; d1++) {
+				if (contained_in(&pp->pp_data.domain_array[d],&pp->pp_data.domain_array[d1],sublinkage))
+				{
+					pp->pp_data.domain_array[d].parent = &pp->pp_data.domain_array[d1];
+					break;
+				}
+			}
+			if (d1 == pp->pp_data.N_domains) {
+				/* we know this domain is a root of a new tree */
+				pp->pp_data.domain_array[d].parent = NULL;
 			}
-		}
-		if (d1 == pp->pp_data.N_domains) {
-			/* we know this domain is a root of a new tree */
-			pp->pp_data.domain_array[d].parent = NULL;
-			/* It's now ok for this to happen.	It used to do:
-		 printf("I can't find a parent domain for this domain\n");
-		 print_domain(d);
-		 exit(1); */
 		}
 	}
 	/* the parent links of domain nodes have been established.
@@ -796,7 +800,7 @@ static void build_domain_forest(Postprocessor *pp, Sublinkage *sublinkage)
 		pp->pp_data.domain_array[d].child = NULL;
 	}
 	for (link=0; link < sublinkage->num_links; link++) {
-		if (sublinkage->link[link]->l == -1) continue; /* probably not necessary */
+		assert (sublinkage->link[link]->lw != SIZE_MAX);
 		for (d=0; d<pp->pp_data.N_domains; d++) {
 			if (link_in_domain(link, &pp->pp_data.domain_array[d])) {
 				dtl = (DTreeLeaf *) xalloc(sizeof(DTreeLeaf));
@@ -813,7 +817,7 @@ static void build_domain_forest(Postprocessor *pp, Sublinkage *sublinkage)
 static int
 internal_process(Postprocessor *pp, Sublinkage *sublinkage, const char **msg)
 {
-	int i;
+	size_t i;
 	/* quick test: try applying just the relevant global rules */
 	if (!apply_relevant_rules(pp,apply_contains_one_globally,
 								sublinkage,
@@ -839,7 +843,7 @@ internal_process(Postprocessor *pp, Sublinkage *sublinkage, const char **msg)
 #endif
 
 	/* The order below should be optimal for most cases */
-	if (!apply_relevant_rules(pp,apply_contains_one, sublinkage,
+	if (!apply_relevant_rules(pp, apply_contains_one, sublinkage,
 								pp->knowledge->contains_one_rules,
 								pp->relevant_contains_one_rules, msg))	return 1;
 	if (!apply_relevant_rules(pp,apply_contains_none, sublinkage,
@@ -977,13 +981,14 @@ void post_process_scan_linkage(Postprocessor *pp, Parse_Options opts,
 									 Sentence sent, Sublinkage *sublinkage)
 {
 	const char *p;
-	int i;
+	size_t i;
 	if (pp == NULL) return;
 	if (sent->length < opts->twopass_length) return;
 	for (i=0; i<sublinkage->num_links; i++)
 	{
-		if (sublinkage->link[i]->l == -1) continue;
-		p = string_set_add(sublinkage->link[i]->name, pp->sentence_link_name_set);
+		// if (sublinkage->link[i]->lw == SIZE_MAX) continue;
+		assert(sublinkage->link[i]->lw != SIZE_MAX);
+		p = string_set_add(sublinkage->link[i]->link_name, pp->sentence_link_name_set);
 		pp_linkset_add(pp->set_of_links_of_sentence, p);
 	}
 }
